##########################################################################################################
##	S-score calculation
##	Created by Jialiang Huang (huangjialiangcn@gmail.com or jialiang.j.huang@gsk.com) 2012-11-23
##	(See Jialiang Huang et. al 2012 Systematic Prediction of Pharmacodynamic Drug-Drug Interactions through Protein-Protein-Interaction Network. for more detail)
##	
##########################################################################################################
use strict;
use Data::Dumper;
use Time::localtime;
my $time;
my $MEAN_PCC = 0.425;				##	Backgroud of pcc = 0.425

my(	$DrugTargetFile,	##	A tab-delimited file provides Drug-Target	
	$NetworkFile,		##	A tab-delimited file provides Network info.
	$OutputFile)		##	A tab-delimited file for output
	= GetOptions();
Sscore($DrugTargetFile,$NetworkFile,$OutputFile);

##	get options from command
##	command: perl Sscore.pl -t DrugBank.FDAapproved.DrugTargetAssociations.txt -n HPRD.Weighted.txt -o S-score.txt
sub GetOptions{
	my %parameters = @ARGV;	
	my $DrugTargetFile = '';
	my $NetworkFile = '';
	my $OutputFile = './output.txt';
	
	##	-t
	if(exists $parameters{'-t'} && defined $parameters{'-t'}){
		$DrugTargetFile = $parameters{'-t'} 
	}else{
		print "ERROR:\tNOT DrugTargetFile\n";
		usage();
	}
	
	##	-n
	if(exists $parameters{'-n'} && defined $parameters{'-n'}){
		$NetworkFile = $parameters{'-n'} 
	}else{
		print "ERROR:\tNOT NetworkFile\n";
		usage();
	}
	
	##	-o
	if(exists $parameters{'-o'} && defined $parameters{'-o'}){
		$OutputFile = $parameters{'-o'} 
	}
	
	##	@ARGV
	usage() if(@ARGV<1);
	##	-h
	usage() if(exists $parameters{'-h'});
	
	return($DrugTargetFile,$NetworkFile,$OutputFile);
}


####################### USAGE;
sub usage {
    print <<"END_USAGE";
Usage:
perl Sscore.pl -t <DrugTargetFile> -n <NetworkFile> -o [OutputFile] -h
-t  DrugTargetFile for Drug-Target [required]
-n  NetworkFile for Network info. [required]
-o  OutputFile for output files (default './output.txt')
-h  help

END_USAGE
    exit;
}

sub Sscore{
	my($DrugTargetFile,$NetworkFile,$OutputFile) = @_;
	
#	my $fileIn1 = './DrugBank.FDA-approved.Drug-Target Associations.txt';					##	
#	my $fileIn2 = './HPRD.Weighted.txt';		##	
#	my $fileOut = './S-score.txt';
	
	my %drug2target = ();				##	$drug2target->{$drug}{$target} = 1
	my %edges = ();						##	$edges->{$gene1}{$gene2}{'Type'|'Weight'}
	my %edgesLoop = ();					##	$edgesLoop->{$gene1}{$gene2} = PCC
	my %drug2centeredSystem = ();		##	$drug2centeredSystem->{$drug}{$gene} = 1
	my %drug2drug = ();					##	%drug2drug{$d1}{$d2} = (d1&&d2)/(d1||d2)	
	readPair($DrugTargetFile,\%drug2target);
	readPair($NetworkFile,\%edges);
	
	GetLoopEdge(\%edges,\%edgesLoop);
	DefineTargetCenteredSystem(\%drug2target,\%edgesLoop,\%drug2centeredSystem);
	CalculateSscore(\%drug2centeredSystem,\%edgesLoop,\%drug2drug);
	
	writePair($OutputFile,\%drug2drug);	
}

##	get loop edges
##	$edges->{$g1}{$g2}{$item} => edgesLoop->{$g1}{$g2}{$item}, $edgesLoop->{$g2}{$g1}{$item}
sub GetLoopEdge{
	my($edges,$edgesLoop) = @_;
	
	foreach my $g1(keys %{$edges}){
		foreach my $g2(keys %{$edges->{$g1}}){
			$edgesLoop->{$g1}{$g2} = $edges->{$g1}{$g2};
			$edgesLoop->{$g2}{$g1} = $edges->{$g1}{$g2};
		}
	}
	
	##	add self edge pcc=1
	foreach my $g(keys %{$edgesLoop}){
		$edgesLoop->{$g}{$g} = 1;
	}
}

##	Defined a target-centered system for each drug, which includes drug targets and their first-step neighboring proteins in the PPI network  
sub DefineTargetCenteredSystem{
	my ($drug2target,$edgesLoop,$drug2centeredSystem) = @_;
	
	##	Add target to TargetCenteredSystem
	foreach my $drug(keys %{$drug2target}){
		foreach my $target(keys %{$drug2target->{$drug}}){
			$drug2centeredSystem->{$drug}{$target} = 'Target';
		}
	}
	
	##	Add Target's first-step neighbor protein to TargetCenteredSystem 
	foreach my $drug(keys %{$drug2target}){
		foreach my $target(keys %{$drug2target->{$drug}}){
			if(defined $edgesLoop->{$target}){
				foreach my $neighbor(keys %{$edgesLoop->{$target}}){
					$drug2centeredSystem->{$drug}{$neighbor} = 'Neighbor' if(!defined $drug2centeredSystem->{$neighbor});
				}
			}
		}
	}	
}

##	Defined a system connection score (S-score) to describe the connection between two target-centered systems in the PPI network as the following (Figure 2D): 
##	S-score = (x-u)/(s/sqrt(n)), 
##	where x,s and n represent the mean, standard deviation and number of the cross-tissue expression PCC of edges connecting two drug-centered systems, respectively; 
##			u represents the average PCC of all edges in the network as background. 
sub CalculateSscore{
	my ($drug2centeredSystem,$edgesLoop,$drug2drug) = @_;
	
	my @drugs = sort keys %{$drug2centeredSystem};
	my $numDrug = @drugs;
	
	for(my $i=0;$i<@drugs-1;$i++){
		print $time = ctime(),"\tDrugs:\t$i:$numDrug...\n";
		for(my $j=$i+1;$j<@drugs;$j++){
			##	connection pcc
			my @connection = ();
			my $idx = 0;
			foreach my $g1(keys %{$drug2centeredSystem->{$drugs[$i]}}){
				foreach my $g2(keys %{$drug2centeredSystem->{$drugs[$j]}}){
					if(defined $edgesLoop->{$g1}{$g2}){
						$connection[$idx] = abs($edgesLoop->{$g1}{$g2});
						$idx++;
					}				
				}
			}
			$connection[$idx] = $MEAN_PCC;		##	add mean_pcc of background in case only one value			
			##	connection pcc stat
			if(@connection>=2){
				my %stat = ();
				GetStatistics(\@connection,\%stat);
				my $sscore = ($stat{'mean'} - $MEAN_PCC)/sqrt($stat{'variance'}/$stat{'count'}); 	##	S-score
				$drug2drug->{$drugs[$i]}{$drugs[$j]} = $sscore if($sscore>0);						##	Only keep positive value
			}						
		}
	}
}

##	calculate Statistics: x,s and n 
sub GetStatistics{
	my($data,$statistics) = @_;
	
	##	count
	my $count = @{$data};
	##	mean
	my $mean = 0;
	for(my $i=0;$i<$count;$i++){
		$mean += $data->[$i];
	}
	$mean /= $count if($count !=0);
	##	variance
	my $variance = 0;
	for(my $i=0;$i<$count;$i++){
		$variance += ($data->[$i] - $mean)*($data->[$i] - $mean);
	}
	$variance /= $count if($count !=0);

	$statistics->{'count'} = $count;
	$statistics->{'mean'} = $mean;
	$statistics->{'variance'} = $variance;
}

##	Read/Write files
#########################################
##	static method:	Reads a file in table format 
##	input:	
##		$file			==>	the name of the file which the data are to be read from:(a,b,value);
##	output:
##		$records		==>	the ref which point to the data
##	return:
##		if success return 1;
######################################### 
sub readPair{
	my ($file,$records,$skip) = @_;
	print "Reading file:\t$file...\n";
	open(IN,"$file")|| die "can't not open the file : $!";	
	##	skip lines
	if(!defined $skip){
		$skip = 1;
	}
	while($skip){
		<IN>;
		print "Note: The first $skip lines in file $file have been skip!\n" if($skip >= 2);	
		$skip--;	
	}
	
	my $count = 0;
	while(<IN>){
		$count++;
		print "reading record line:\t$count\n" if($count %10000 ==0);
		chomp();
		my @items = split(/\t/,$_);
		my $numItems = @items;
		if($numItems>=3){
			$records->{$items[0]}{$items[1]} = $items[2];
		}elsif($numItems == 2){
			$records->{$items[0]}{$items[1]} = 1;
		}elsif($numItems == 1){
			$records->{$1}{'0'} = 1;
		}else{
			die "the number of records in line $count is NULL!\n";
		}
	}

	##	return reading info
	my %X;
	my %Y;
	my $numxy = 0;
	foreach my $x(keys %{$records}){
		foreach my $y(keys %{$records->{$x}}){
			$X{$x}++;
			$Y{$y}++;
			$numxy++;
		}
	}
	my $numx = keys %X;
	my $numy = keys %Y;
	print "x:\t$numx\ty:\t$numy\txy:\t$numxy\n";
	
	close(IN);
	return 1;
}

#########################################
##	static method:	Reads a file in table format 
##	input:	
##		$records		==>	$records{$x}{$y} = 1
##	output:
##		$file			==>	($x,$y);
##	return:
##		if success return 1;
#########################################
sub writePair{
	my ($file,$records) = @_;
	print "Writing file:\t$file...\n";
	open(OUT,">$file")|| die "can't not open the file : $!";
	
	print OUT "DrugBankID 1\tDrugBankID 2\tS-score\n";
	
	foreach my $x(sort {$a<=>$b}keys %{$records}){
		foreach my $y(sort {$a<=>$b} keys %{$records->{$x}}){
			print OUT "$x\t$y\t$records->{$x}{$y}\n";
		}
	}
	close(OUT);	
	return 1;
}


